@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -30,29 +30,43 @@
 #include "../../dprint.h"
 #include "../../mod_fix.h"
 #include "../../pvar.h"
+#include "../../usr_avp.h"
 #include "../../parser/sdp/sdp.h"
+#include "../../parser/sdp/sdp_helpr_funcs.h"
 #include "../../trim.h"
 #include "../../data_lump.h"
+#include "../../ut.h"
 
 #include "api.h"
 #include "sdpops_data.h"
 
 MODULE_VERSION
 
+static int w_sdp_remove_line_by_prefix(sip_msg_t* msg, char* prefix, char* bar);
 static int w_sdp_remove_codecs_by_id(sip_msg_t* msg, char* codecs, char *bar);
 static int w_sdp_remove_codecs_by_name(sip_msg_t* msg, char* codecs, char *bar);
 static int w_sdp_keep_codecs_by_id(sip_msg_t* msg, char* codecs, char *bar);
 static int w_sdp_keep_codecs_by_name(sip_msg_t* msg, char* codecs, char *bar);
 static int w_sdp_with_media(sip_msg_t* msg, char* media, char *bar);
+static int w_sdp_with_transport(sip_msg_t* msg, char* transport, char *bar);
+static int w_sdp_with_transport_like(sip_msg_t* msg, char* transport, char *bar);
+static int w_sdp_transport(sip_msg_t* msg, char *bar);
 static int w_sdp_with_codecs_by_id(sip_msg_t* msg, char* codec, char *bar);
 static int w_sdp_with_codecs_by_name(sip_msg_t* msg, char* codec, char *bar);
 static int w_sdp_remove_media(sip_msg_t* msg, char* media, char *bar);
+static int w_sdp_remove_transport(sip_msg_t* msg, char* transport, char *bar);
 static int w_sdp_print(sip_msg_t* msg, char* level, char *bar);
-static int w_get_sdp(sip_msg_t* msg, char *bar);
+static int w_sdp_get(sip_msg_t* msg, char *bar);
+static int w_sdp_content(sip_msg_t* msg, char* foo, char *bar);
+static int w_sdp_with_ice(sip_msg_t* msg, char* foo, char *bar);
+static int w_sdp_get_line_startswith(sip_msg_t* msg, char *foo, char *bar);
+
 
 static int mod_init(void);
 
 static cmd_export_t cmds[] = {
+	{"sdp_remove_line_by_prefix",  (cmd_function)w_sdp_remove_line_by_prefix,
+		1, fixup_spve_null,  0, ANY_ROUTE},
 	{"sdp_remove_codecs_by_id",    (cmd_function)w_sdp_remove_codecs_by_id,
 		1, fixup_spve_null,  0, ANY_ROUTE},
 	{"sdp_remove_codecs_by_name",  (cmd_function)w_sdp_remove_codecs_by_name,
@@ -69,14 +83,28 @@ static cmd_export_t cmds[] = {
 		1, fixup_spve_null,  0, ANY_ROUTE},
 	{"sdp_remove_media",             (cmd_function)w_sdp_remove_media,
 		1, fixup_spve_null,  0, ANY_ROUTE},
+	{"sdp_with_transport",         (cmd_function)w_sdp_with_transport,
+		1, fixup_spve_null,  0, ANY_ROUTE},
+	{"sdp_with_transport_like",  (cmd_function)w_sdp_with_transport_like,
+		1, fixup_spve_null,  0, ANY_ROUTE},
+	{"sdp_remove_transport",       (cmd_function)w_sdp_remove_transport,
+		1, fixup_spve_null,  0, ANY_ROUTE},
+	{"sdp_transport",              (cmd_function)w_sdp_transport,
+		1, 0,  0, ANY_ROUTE},
 	{"sdp_with_codecs_by_id",      (cmd_function)w_sdp_with_codecs_by_id,
 		1, fixup_spve_null,  0, ANY_ROUTE},
 	{"sdp_with_codecs_by_name",    (cmd_function)w_sdp_with_codecs_by_name,
 		1, fixup_spve_null,  0, ANY_ROUTE},
 	{"sdp_print",                  (cmd_function)w_sdp_print,
 		1, fixup_igp_null,  0, ANY_ROUTE},
-	{"sdp_get",                  (cmd_function)w_get_sdp,
+	{"sdp_get",                  (cmd_function)w_sdp_get,
 		1, 0,  0, ANY_ROUTE},
+	{"sdp_content",                (cmd_function)w_sdp_content,
+		0, 0,  0, ANY_ROUTE},
+	{"sdp_with_ice",                (cmd_function)w_sdp_with_ice,
+		0, 0,  0, ANY_ROUTE},
+	{"sdp_get_line_startswith", (cmd_function)w_sdp_get_line_startswith,
+		2, 0,  0, ANY_ROUTE},
 	{"bind_sdpops",                (cmd_function)bind_sdpops,
 		1, 0, 0, 0},
 	{0, 0, 0, 0, 0, 0}
@@ -128,12 +156,17 @@ static int mod_init(void)
 int sdp_locate_line(sip_msg_t* msg, char *pos, str *aline)
 {
 	char *p;
+	char *bend;
+
 	p = pos;
 	while(*p!='\n') p--;
 	aline->s = p + 1;
 	p = pos;
-	while(*p!='\n') p++;
+	bend = msg->buf+msg->len;
+	while(*p!='\n' && p<bend) p++;
 	aline->len = p - aline->s + 1;
+	if(unlikely(p==bend)) aline->len--;
+
 	return 0;
 }
 
@@ -150,8 +183,8 @@ int sdp_remove_str_codec_id_attrs(sip_msg_t* msg,
 	payload = sdp_stream->payload_attr;
 	while (payload) {
 		LM_DBG("a= ... for codec %.*s/%.*s\n",
-			payload->rtp_payload.len, payload->rtp_payload.s,
-			payload->rtp_enc.len, payload->rtp_enc.s);
+				payload->rtp_payload.len, payload->rtp_payload.s,
+				payload->rtp_enc.len, payload->rtp_enc.s);
 		if(rm_codec->len==payload->rtp_payload.len
 				&& strncmp(payload->rtp_payload.s, rm_codec->s,
 					rm_codec->len)==0) {
@@ -163,8 +196,8 @@ int sdp_remove_str_codec_id_attrs(sip_msg_t* msg,
 							aline.len, 0);
 					if (anchor == NULL) {
 						LM_ERR("failed to remove [%.*s] inside [%.*s]\n",
-							rm_codec->len, rm_codec->s,
-							aline.len, aline.s);
+								rm_codec->len, rm_codec->s,
+								aline.len, aline.s);
 						return -1;
 					}
 				}
@@ -177,8 +210,8 @@ int sdp_remove_str_codec_id_attrs(sip_msg_t* msg,
 							aline.len, 0);
 					if (anchor == NULL) {
 						LM_ERR("failed to remove [%.*s] inside [%.*s]\n",
-							rm_codec->len, rm_codec->s,
-							aline.len, aline.s);
+								rm_codec->len, rm_codec->s,
+								aline.len, aline.s);
 						return -1;
 					}
 				}
@@ -208,7 +241,7 @@ int sdp_codec_in_str(str *allcodecs, str* codec, char delim)
 			if(codec->len <= allcodecs->len-i) {
 				if(strncmp(&allcodecs->s[i], codec->s, codec->len)==0) {
 					if(&allcodecs->s[i+codec->len]
-									== &allcodecs->s[allcodecs->len]
+							== &allcodecs->s[allcodecs->len]
 							|| allcodecs->s[i+codec->len] == delim) {
 						/* match */
 						return 1;
@@ -245,12 +278,12 @@ int sdp_remove_str_codec_id(sip_msg_t* msg, str *allcodecs, str* rmcodec)
 			if(rmcodec->len <= allcodecs->len-i) {
 				if(strncmp(&allcodecs->s[i], rmcodec->s, rmcodec->len)==0) {
 					if(&allcodecs->s[i+rmcodec->len]
-									== &allcodecs->s[allcodecs->len]
+							== &allcodecs->s[allcodecs->len]
 							|| allcodecs->s[i+rmcodec->len] == ' ') {
 						/* match - remove also the space before codec id */
 						LM_DBG("found codec [%.*s] inside [%.*s]\n",
-									rmcodec->len, rmcodec->s,
-									allcodecs->len, allcodecs->s);
+								rmcodec->len, rmcodec->s,
+								allcodecs->len, allcodecs->s);
 						anchor = del_lump(msg, &allcodecs->s[i-1] - msg->buf,
 								rmcodec->len+1, 0);
 						if (anchor == NULL) {
@@ -314,8 +347,8 @@ int sdp_remove_codecs_by_id(sip_msg_t* msg, str* codecs)
 			if(!sdp_stream) break;
 
 			LM_DBG("stream %d of %d - payloads [%.*s]\n",
-				sdp_stream_num, sdp_session_num, 
-				sdp_stream->payloads.len, sdp_stream->payloads.s);
+					sdp_stream_num, sdp_session_num, 
+					sdp_stream->payloads.len, sdp_stream->payloads.s);
 			sdp_codecs = sdp_stream->payloads;
 			tmp_codecs = *codecs;
 			while(str_find_token(&tmp_codecs, &rm_codec, ',')==0
@@ -338,6 +371,126 @@ int sdp_remove_codecs_by_id(sip_msg_t* msg, str* codecs)
 	return 0;
 }
 
+// removes consecutive blocks of SDP lines that begin with script provided prefix
+int sdp_remove_line_by_prefix(sip_msg_t* msg, str* prefix)
+{
+	str body = {NULL, 0};
+	str remove = {NULL, 0};
+	str line = {NULL, 0};
+	char* del_lump_start = NULL;
+	char* del_lump_end = NULL;
+	int del_lump_flag = 0;
+	struct lump *anchor;
+	char* p = NULL;
+
+	if(parse_sdp(msg) < 0) {
+		LM_ERR("Unable to parse sdp\n");
+		return -1;
+	}
+
+	if(msg->body == NULL) {
+		LM_DBG("No sdp body\n");
+		return -1;
+	}
+
+	body.s = ((sdp_info_t*)msg->body)->raw_sdp.s;
+	body.len = ((sdp_info_t*)msg->body)->raw_sdp.len;
+
+	if (body.s==NULL) {
+		LM_ERR("failed to get the message body\n");
+		return -1;
+	}
+	body.len = msg->len - (body.s - msg->buf);
+	if (body.len==0) {
+		LM_DBG("message body has zero length\n");
+		return -1;
+	}
+
+	p = find_sdp_line(body.s, body.s+body.len, prefix->s[0]);
+	while (p != NULL)
+	{
+		if (sdp_locate_line(msg, p, &line) != 0)
+		{
+			LM_ERR("sdp_locate_line fail\n");
+			return -1;
+		}
+
+		//LM_DBG("line.s: %.*s\n", line.len, line.s);
+
+		if (extract_field(&line, &remove, *prefix) == 0)
+		{
+			//LM_DBG("line range: %d - %d\n", line.s - body.s, line.s + line.len - body.s);
+
+			if (del_lump_start == NULL)
+			{
+				del_lump_start = line.s;
+				del_lump_end = line.s + line.len;
+				//LM_DBG("first match, prepare new lump  (len=%d)\n", line.len);
+			}
+			else if ( p == del_lump_end )  // current line is same as del_lump_end
+			{
+				del_lump_end = line.s + line.len;
+				//LM_DBG("cont. match, made lump longer  (len+=%d)\n", line.len);
+			}
+
+			if (del_lump_end >= body.s + body.len)
+			{
+				//LM_DBG("end of buffer, delete lump\n");
+				del_lump_flag = 1;
+			}
+			//LM_DBG("lump pos: %d - %d\n", del_lump_start - body.s, del_lump_end - body.s);
+		}
+		else if ( del_lump_end != NULL)
+		{
+			//LM_DBG("line does not start with search pattern, delete current lump\n");
+			del_lump_flag = 1;
+		}
+
+		if (del_lump_flag && del_lump_start && del_lump_end)
+		{
+			LM_DBG("del_lump range: %d - %d  len: %d\n", (int)(del_lump_start - body.s),
+					(int)(del_lump_end - body.s), (int)(del_lump_end - del_lump_start));
+
+			anchor = del_lump(msg, del_lump_start - msg->buf, del_lump_end - del_lump_start, HDR_OTHER_T);
+			if (anchor == NULL)
+			{
+				LM_ERR("failed to remove lump\n");
+				return -1;
+			}
+
+			del_lump_start = NULL;
+			del_lump_end = NULL;
+			del_lump_flag = 0;
+			//LM_DBG("succesful lump deletion\n");
+		}
+
+		p = find_sdp_line(line.s + line.len, body.s + body.len, prefix->s[0]);
+	}
+	return 0;
+}
+
+static int w_sdp_remove_line_by_prefix(sip_msg_t* msg, char* prefix, char* bar)
+{
+	str prfx = {NULL, 0};
+
+	if(prefix==0)
+	{
+		LM_ERR("invalid parameters\n");
+		return -1;
+	}
+
+	if (get_str_fparam(&prfx, msg, (fparam_t*)prefix))
+	{
+		LM_ERR("unable to determine prefix\n");
+		return -1;
+	}
+	LM_DBG("Removing SDP lines with prefix: %.*s\n", prfx.len, prfx.s);
+
+	if(sdp_remove_line_by_prefix(msg, &prfx)<0)
+		return -1;
+	return 1;
+}
+
 /**
  *
  */
@@ -460,8 +613,8 @@ int sdp_keep_codecs_by_id(sip_msg_t* msg, str* codecs, str *media)
 			if(!sdp_stream) break;
 
 			LM_DBG("stream %d of %d - payloads [%.*s]\n",
-				sdp_stream_num, sdp_session_num,
-				sdp_stream->payloads.len, sdp_stream->payloads.s);
+					sdp_stream_num, sdp_session_num,
+					sdp_stream->payloads.len, sdp_stream->payloads.s);
 			if((media==NULL)
 					|| (media->len==sdp_stream->media.len
 						&& strncasecmp(sdp_stream->media.s, media->s,
@@ -477,8 +630,8 @@ int sdp_keep_codecs_by_id(sip_msg_t* msg, str* codecs, str *media)
 
 					if(sdp_codec_in_str(codecs, &rm_codec, ',')==0) {
 						LM_DBG("codecs [%.*s] - remove [%.*s]\n",
-							sdp_codecs.len, sdp_codecs.s,
-							rm_codec.len, rm_codec.s);
+								sdp_codecs.len, sdp_codecs.s,
+								rm_codec.len, rm_codec.s);
 						sdp_remove_str_codec_id(msg, &sdp_codecs, &rm_codec);
 						sdp_remove_str_codec_id_attrs(msg, sdp_stream, &rm_codec);
 					}
@@ -623,11 +776,11 @@ static int sdp_with_media(sip_msg_t *msg, str *media)
 			if(!sdp_stream) break;
 
 			LM_DBG("stream %d of %d - media [%.*s]\n",
-				sdp_stream_num, sdp_session_num,
-				sdp_stream->media.len, sdp_stream->media.s);
+					sdp_stream_num, sdp_session_num,
+					sdp_stream->media.len, sdp_stream->media.s);
 			if(media->len==sdp_stream->media.len
 					&& strncasecmp(sdp_stream->media.s, media->s,
-							media->len)==0)
+						media->len)==0)
 				return 1;
 			sdp_stream_num++;
 		}
@@ -700,16 +853,16 @@ static int sdp_remove_media(sip_msg_t *msg, str *media)
 			if(!sdp_stream) break;
 
 			LM_DBG("stream %d of %d - media [%.*s]\n",
-				sdp_stream_num, sdp_session_num,
-				sdp_stream->media.len, sdp_stream->media.s);
+					sdp_stream_num, sdp_session_num,
+					sdp_stream->media.len, sdp_stream->media.s);
 			if(media->len==sdp_stream->media.len
 					&& strncasecmp(sdp_stream->media.s, media->s,
-							media->len)==0)
+						media->len)==0)
 			{
 				/* found - remove */
 				LM_DBG("removing media stream: %.*s", media->len, media->s);
 				nxt_stream = get_sdp_stream(msg, sdp_session_num,
-								sdp_stream_num+1);
+						sdp_stream_num+1);
 				/* skip back 'm=' */
 				dstart = sdp_stream->media.s - 2;
 				if(!nxt_stream) {
@@ -722,7 +875,7 @@ static int sdp_remove_media(sip_msg_t *msg, str *media)
 				anchor = del_lump(msg, dstart - msg->buf, dlen, 0);
 				if (anchor == NULL) {
 					LM_ERR("failed to remove media type [%.*s]\n",
-						 media->len, media->s);
+							media->len, media->s);
 					return -1;
 				}
 
@@ -762,6 +915,285 @@ static int w_sdp_remove_media(sip_msg_t* msg, char* media, char *bar)
 }
 
 
+/** 
+ * @brief check 'media' matches the value of any 'm=media port value ...' lines
+ * @return -1 - error; 0 - not found; 1 - found
+ */
+static int sdp_with_transport(sip_msg_t *msg, str *transport, int like)
+{
+	int sdp_session_num;
+	int sdp_stream_num;
+	sdp_session_cell_t* sdp_session;
+	sdp_stream_cell_t* sdp_stream;
+
+	if(parse_sdp(msg) < 0) {
+		LM_ERR("Unable to parse sdp\n");
+		return -1;
+	}
+
+	LM_DBG("attempting to search for transport type: [%.*s]\n",
+			transport->len, transport->s);
+
+	sdp_session_num = 0;
+	for(;;)
+	{
+		sdp_session = get_sdp_session(msg, sdp_session_num);
+		if(!sdp_session) break;
+		sdp_stream_num = 0;
+		for(;;)
+		{
+			sdp_stream = get_sdp_stream(msg, sdp_session_num, sdp_stream_num);
+			if(!sdp_stream) break;
+
+			LM_DBG("stream %d of %d - transport [%.*s]\n",
+					sdp_stream_num, sdp_session_num,
+					sdp_stream->transport.len, sdp_stream->transport.s);
+			if (like == 0) {
+			    if(transport->len==sdp_stream->transport.len
+			       && strncasecmp(sdp_stream->transport.s, transport->s,
+					      transport->len)==0)
+				return 1;
+			} else {
+			    if (ser_memmem(sdp_stream->transport.s, transport->s,
+					   sdp_stream->transport.len, transport->len)!=NULL)
+				return 1;
+			}
+			sdp_stream_num++;
+		}
+		sdp_session_num++;
+	}
+
+	return 0;
+}
+
+/** 
+ * @brief assigns common media transport (if any) of 'm' lines to pv argument
+ * @return -1 - error; 0 - not found; 1 - found
+ */
+static int w_sdp_transport(sip_msg_t* msg, char *avp)
+{
+    int_str avp_val;
+    int_str avp_name;
+    static unsigned short avp_type = 0;
+    str s;
+    pv_spec_t *avp_spec = NULL;
+    int sdp_session_num;
+    int sdp_stream_num;
+    sdp_session_cell_t* sdp_session;
+    sdp_stream_cell_t* sdp_stream;
+    str *transport;
+
+    s.s = avp; s.len = strlen(s.s);
+    if (pv_locate_name(&s) != s.len) {
+	LM_ERR("invalid avp parameter %s\n", avp);
+	return -1;
+    }
+    if (((avp_spec = pv_cache_get(&s)) == NULL)
+	|| avp_spec->type!=PVT_AVP) {
+	LM_ERR("malformed or non AVP %s\n", avp);
+	return -1;
+    }
+    if (pv_get_avp_name(0, &avp_spec->pvp, &avp_name, &avp_type) != 0) {
+	LM_ERR("invalid AVP definition %s\n", avp);
+	return -1;
+    }
+
+    if(parse_sdp(msg) < 0) {
+	LM_ERR("unable to parse sdp\n");
+	return -1;
+    }
+
+    sdp_session_num = 0;
+    transport = (str *)NULL;
+
+    for (;;) {
+	sdp_session = get_sdp_session(msg, sdp_session_num);
+	if (!sdp_session) break;
+	sdp_stream_num = 0;
+	for (;;) {
+	    sdp_stream = get_sdp_stream(msg, sdp_session_num,
+					sdp_stream_num);
+	    if (!sdp_stream) break;
+	    LM_DBG("stream %d of %d - transport [%.*s]\n",
+		   sdp_stream_num, sdp_session_num,
+		   sdp_stream->transport.len, sdp_stream->transport.s);
+	    if (transport) {
+		if (transport->len != sdp_stream->transport.len
+		    || strncasecmp(sdp_stream->transport.s, transport->s,
+				   transport->len) != 0) {
+		    LM_DBG("no common transport\n");
+		    return -2;
+		}
+	    } else {
+		transport = &sdp_stream->transport;
+	    }
+	    sdp_stream_num++;
+	}
+	sdp_session_num++;
+    }
+    if (transport) {
+	avp_val.s.s = transport->s;
+	avp_val.s.len = transport->len;
+	LM_DBG("found common transport '%.*s'\n",
+	       transport->len, transport->s);
+	if (add_avp(AVP_VAL_STR | avp_type, avp_name, avp_val) != 0) {
+	    LM_ERR("failed to add transport avp");
+	    return -1;
+	}
+    }
+
+    return 1;
+}
+
+
+/**
+ *
+ */
+static int w_sdp_with_transport(sip_msg_t* msg, char* transport, char *bar)
+{
+	str ltransport = {0, 0};
+
+	if(transport==0)
+	{
+		LM_ERR("invalid parameters\n");
+		return -1;
+	}
+
+	if(fixup_get_svalue(msg, (gparam_p)transport, &ltransport)!=0)
+	{
+		LM_ERR("unable to get the transport value\n");
+		return -1;
+	}
+
+	if(sdp_with_transport(msg, &ltransport, 0)<=0)
+		return -1;
+	return 1;
+}
+
+/**
+ *
+ */
+static int w_sdp_with_transport_like(sip_msg_t* msg, char* transport, char *bar)
+{
+	str ltransport = {0, 0};
+
+	if(transport==0)
+	{
+		LM_ERR("invalid parameters\n");
+		return -1;
+	}
+
+	if(fixup_get_svalue(msg, (gparam_p)transport, &ltransport)!=0)
+	{
+		LM_ERR("unable to get the transport value\n");
+		return -1;
+	}
+
+	if(sdp_with_transport(msg, &ltransport, 1)<=0)
+		return -1;
+	return 1;
+}
+
+/**
+ * @brief remove streams matching the m=media port 'transport'
+ * @return -1 - error; 0 - not found; >=1 - found
+ */
+static int sdp_remove_transport(sip_msg_t *msg, str *transport)
+{
+	sdp_info_t *sdp = NULL;
+	int sdp_session_num;
+	int sdp_stream_num;
+	sdp_session_cell_t* sdp_session;
+	sdp_stream_cell_t* sdp_stream;
+	sdp_stream_cell_t* nxt_stream;
+	int ret = 0;
+	char *dstart = NULL;
+	int dlen = 0;
+	struct lump *anchor;
+
+	if(parse_sdp(msg) < 0) {
+		LM_ERR("Unable to parse sdp\n");
+		return -1;
+	}
+
+	LM_DBG("attempting to search for transport type: [%.*s]\n",
+			transport->len, transport->s);
+
+	sdp = (sdp_info_t*)msg->body;
+
+	sdp_session_num = 0;
+	for(;;)
+	{
+		sdp_session = get_sdp_session(msg, sdp_session_num);
+		if(!sdp_session) break;
+		sdp_stream_num = 0;
+		for(;;)
+		{
+			sdp_stream = get_sdp_stream(msg, sdp_session_num, sdp_stream_num);
+			if(!sdp_stream) break;
+
+			LM_DBG("stream %d of %d - transport [%.*s]\n",
+					sdp_stream_num, sdp_session_num,
+					sdp_stream->transport.len, sdp_stream->transport.s);
+			if(transport->len==sdp_stream->transport.len
+					&& strncasecmp(sdp_stream->transport.s, transport->s,
+						transport->len)==0)
+			{
+				/* found - remove */
+				LM_DBG("removing transport stream: %.*s", transport->len, transport->s);
+				nxt_stream = get_sdp_stream(msg, sdp_session_num,
+						sdp_stream_num+1);
+				/* skip back 'm=' */
+				dstart = sdp_stream->media.s - 2;
+				if(!nxt_stream) {
+					/* delete to end of sdp */
+					dlen = (int)(sdp->text.s + sdp->text.len - dstart);
+				} else {
+					/* delete to start of next stream */
+					dlen = (int)(nxt_stream->media.s - 2 - dstart);
+				}
+				anchor = del_lump(msg, dstart - msg->buf, dlen, 0);
+				if (anchor == NULL) {
+					LM_ERR("failed to remove transport type [%.*s]\n",
+							transport->len, transport->s);
+					return -1;
+				}
+
+				ret++;
+			}
+			sdp_stream_num++;
+		}
+		sdp_session_num++;
+	}
+
+	return ret;
+}
+
+
+/**
+ *
+ */
+static int w_sdp_remove_transport(sip_msg_t* msg, char* transport, char *bar)
+{
+	str ltransport = {0, 0};
+
+	if(transport==0)
+	{
+		LM_ERR("invalid parameters\n");
+		return -1;
+	}
+
+	if(fixup_get_svalue(msg, (gparam_p)transport, &ltransport)!=0)
+	{
+		LM_ERR("unable to get the transport value\n");
+		return -1;
+	}
+
+	if(sdp_remove_transport(msg, &ltransport)<=0)
+		return -1;
+	return 1;
+}
+
 /**
  *
  */
@@ -805,8 +1237,8 @@ int sdp_with_codecs_by_id(sip_msg_t* msg, str* codecs)
 			if(!sdp_stream) break;
 
 			LM_DBG("stream %d of %d - payloads [%.*s]\n",
-				sdp_stream_num, sdp_session_num,
-				sdp_stream->payloads.len, sdp_stream->payloads.s);
+					sdp_stream_num, sdp_session_num,
+					sdp_stream->payloads.len, sdp_stream->payloads.s);
 			sdp_codecs = sdp_stream->payloads;
 			tmp_codecs = *codecs;
 			while(str_find_token(&tmp_codecs, &fnd_codec, ',')==0
@@ -817,13 +1249,13 @@ int sdp_with_codecs_by_id(sip_msg_t* msg, str* codecs)
 
 				if(sdp_codec_in_str(&sdp_codecs, &fnd_codec, ' ')==0) {
 					LM_DBG("codecs [%.*s] - not found [%.*s]\n",
-						sdp_codecs.len, sdp_codecs.s,
-						fnd_codec.len, fnd_codec.s);
+							sdp_codecs.len, sdp_codecs.s,
+							fnd_codec.len, fnd_codec.s);
 					notfound = 1;
 				} else {
 					LM_DBG("codecs [%.*s] - found [%.*s]\n",
-						sdp_codecs.len, sdp_codecs.s,
-						fnd_codec.len, fnd_codec.s);
+							sdp_codecs.len, sdp_codecs.s,
+							fnd_codec.len, fnd_codec.s);
 					foundone = 1;
 				}
 			}
@@ -935,7 +1367,7 @@ static int w_sdp_print(sip_msg_t* msg, char* level, char *bar)
 /**
  *
  */
-static int w_get_sdp(sip_msg_t* msg, char *avp)
+static int w_sdp_get(sip_msg_t* msg, char *avp)
 {
 	sdp_info_t *sdp = NULL;
 	int_str avp_val;
@@ -944,7 +1376,7 @@ static int w_get_sdp(sip_msg_t* msg, char *avp)
 	str s;
 	pv_spec_t *avp_spec = NULL;
 	int sdp_missing=1;
-	
+
 	s.s = avp; s.len = strlen(s.s);
 	if (pv_locate_name(&s) != s.len)
 	{
@@ -969,7 +1401,7 @@ static int w_get_sdp(sip_msg_t* msg, char *avp)
 		return -1;
 	}
 	sdp = (sdp_info_t*)msg->body;
-	
+
 	if (sdp_missing) {
 		LM_DBG("No SDP\n");
 		return -2;
@@ -980,13 +1412,171 @@ static int w_get_sdp(sip_msg_t* msg, char *avp)
 	}
 	if (add_avp(AVP_VAL_STR | avp_type, avp_name, avp_val) != 0)
 	{
-	  LM_ERR("Failed to add SDP avp");
-	  return -1;
+		LM_ERR("Failed to add SDP avp");
+		return -1;
 	}
-	
+
 	return 1;
 }
 
+/**
+ *
+ */
+static int w_sdp_content(sip_msg_t* msg, char* foo, char *bar)
+{
+	if(parse_sdp(msg)==0 && msg->body!=NULL)
+		return 1;
+	return -1;
+}
+
+/**
+ *
+ */
+static int w_sdp_with_ice(sip_msg_t* msg, char* foo, char *bar)
+{
+    str ice, body;
+
+    ice.s = "a=candidate";
+    ice.len = 11;
+
+    body.s = get_body(msg);
+    if (body.s == NULL) {
+	LM_DBG("failed to get the message body\n");
+	return -1;
+    }
+
+    body.len = msg->len -(int)(body.s - msg->buf);
+    if (body.len == 0) {
+	LM_DBG("message body has length zero\n");
+	return -1;
+    }
+
+    if (ser_memmem(body.s, ice.s, body.len, ice.len) != NULL) {
+	LM_DBG("found ice attribute\n");
+	return 1;
+    } else {
+	LM_DBG("did't find ice attribute\n");
+	return -1;
+    }
+}
+
+/**
+ *
+ */
+static int w_sdp_get_line_startswith(sip_msg_t *msg, char *avp, char *s_line)
+{
+	sdp_info_t *sdp = NULL;
+	str body = {NULL, 0};
+	str line = {NULL, 0};
+	char* p = NULL;
+	str s;
+	str sline;
+	int_str avp_val;
+	int_str avp_name;
+	pv_spec_t *avp_spec = NULL;
+	static unsigned short avp_type = 0;
+	int sdp_missing=1;
+
+	if (s_line == NULL || strlen(s_line) <= 0)
+	{
+		LM_ERR("Search string is null or empty\n");
+		return -1;
+	}
+	sline.s = s_line;
+	sline.len = strlen(s_line);
+
+	sdp_missing = parse_sdp(msg);
+
+	if(sdp_missing < 0) {
+		LM_ERR("Unable to parse sdp\n");
+		return -1;
+	}
+
+	sdp = (sdp_info_t *)msg->body;
+
+	if (sdp_missing || sdp == NULL)
+	{
+		LM_DBG("No SDP\n");
+		return -2;
+	}
+
+	body.s = sdp->raw_sdp.s;
+	body.len = sdp->raw_sdp.len;
+
+	if (body.s==NULL) {
+		LM_ERR("failed to get the message body\n");
+		return -1;
+	}
+
+	body.len = msg->len - (body.s - msg->buf);
+	if (body.len==0) {
+		LM_DBG("message body has zero length\n");
+		return -1;
+	}
+
+	if (avp == NULL || strlen(avp) <= 0)
+	{
+		LM_ERR("avp variable is null or empty\n");
+		return -1;
+	}
+
+	s.s = avp;
+	s.len = strlen(s.s);
+
+	if (pv_locate_name(&s) != s.len)
+	{
+		LM_ERR("invalid parameter\n");
+		return -1;
+	}
+
+	if (((avp_spec = pv_cache_get(&s)) == NULL)
+			|| avp_spec->type!=PVT_AVP) {
+		LM_ERR("malformed or non AVP %s AVP definition\n", avp);
+		return -1;
+	}
+
+	if(pv_get_avp_name(0, &avp_spec->pvp, &avp_name, &avp_type)!=0)
+	{
+		LM_ERR("[%s]- invalid AVP definition\n", avp);
+		return -1;
+	}
+
+	p = find_sdp_line(body.s, body.s+body.len, sline.s[0]);
+	while (p != NULL)
+	{
+		if (sdp_locate_line(msg, p, &line) != 0)
+		{
+			LM_ERR("sdp_locate_line fail\n");
+			return -1;
+		}
+
+		if (strncmp(line.s, sline.s, sline.len) == 0)
+		{
+			avp_val.s.s = line.s;
+			avp_val.s.len = line.len;
+
+			/* skip ending \r\n if exists */
+			if (avp_val.s.s[line.len-2] == '\r' && avp_val.s.s[line.len-1] == '\n')
+			{
+				/* add_avp() clones to shm and adds 0-terminating char */
+				avp_val.s.len -= 2;
+			}
+
+			if (add_avp(AVP_VAL_STR | avp_type, avp_name, avp_val) != 0)
+			{
+				LM_ERR("Failed to add SDP line avp");
+				return -1;
+			}
+
+			return 1;
+		}
+
+		p = find_sdp_line(line.s + line.len, body.s + body.len, sline.s[0]);
+	}
+
+	return 0;
+}
+
 /**
  *
  */
