@@ -261,7 +261,8 @@ int sql_do_query(sql_con_t *con, str *query, sql_result_t *res)
 	}
 	if(con->dbf.raw_query(con->dbh, query, &db_res)!=0)
 	{
-		LM_ERR("cannot do the query\n");
+		LM_ERR("cannot do the query [%.*s]\n",
+				(query->len>32)?32:query->len, query->s);
 		return -1;
 	}
 
@@ -762,7 +763,7 @@ int sqlops_is_null(str *sres, int i, int j)
 		LM_ERR("row index out of bounds [%d/%d]\n", i, res->nrows);
 		goto error;
 	}
-	if(i>=res->ncols)
+	if(j>=res->ncols)
 	{
 		LM_ERR("column index out of bounds [%d/%d]\n", j, res->ncols);
 		goto error;
