@@ -112,6 +112,8 @@
 #include "t_cancel.h"
 #include "t_lookup.h"
 #include "t_fwd.h"
+#include "t_reply.h"
+#include "h_table.h"
 #include "../../fix_lumps.h"
 #include "config.h"
 #ifdef USE_DNS_FAILOVER
