@@ -22,7 +22,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 /*
  * History:
@@ -97,16 +97,10 @@
 #include "tsend.h"
 #include "forward.h"
 #include "events.h"
-
-#ifdef USE_STUN
-#include "ser_stun.h"
-
-int is_msg_complete(struct tcp_req* r);
-
-#endif /* USE_STUN */
+#include "stun.h"
 
 #ifdef READ_HTTP11
-#define HTTP11CONTINUE	"HTTP/1.1 100 Continue\r\nContent-Lenght: 0\r\n\r\n"
+#define HTTP11CONTINUE	"HTTP/1.1 100 Continue\r\nContent-Length: 0\r\n\r\n"
 #define HTTP11CONTINUE_LEN	(sizeof(HTTP11CONTINUE)-1)
 #endif
 
@@ -123,6 +117,8 @@ static int tcpmain_sock=-1;
 static struct local_timer tcp_reader_ltimer;
 static ticks_t tcp_reader_prev_ticks;
 
+int is_msg_complete(struct tcp_req* r);
+
 /**
  * control cloning of TCP receive buffer
  * - needed for operations working directly inside the buffer
@@ -195,7 +191,7 @@ int tcp_http11_continue(struct tcp_connection *c)
 	{
 		init_dst_from_rcv(&dst, &c->rcv);
 		if (tcp_send(&dst, 0, HTTP11CONTINUE, HTTP11CONTINUE_LEN) < 0) {
-			LOG(L_ERR, "HTTP/1.1 continue failed\n");
+			LM_ERR("HTTP/1.1 continue failed\n");
 		}
 	}
 	/* check for Transfer-Encoding header */
@@ -204,6 +200,16 @@ int tcp_http11_continue(struct tcp_connection *c)
 		c->req.flags |= F_TCP_REQ_BCHUNKED;
 		ret = 1;
 	}
+	/* check for HTTP Via header
+	 * - HTTP Via format is different that SIP Via
+	 * - workaround: replace with Hia to be ignored by SIP parser
+	 */
+	if((p=strfindcasestrz(&msg, "\nVia:"))!=NULL)
+	{
+		p++;
+		*p = 'H';
+		LM_DBG("replaced HTTP Via with Hia [[\n%.*s]]\n", msg.len, msg.s);
+	}
 	return ret;
 }
 #endif /* HTTP11 */
@@ -341,7 +347,7 @@ int tcp_read(struct tcp_connection *c, int* flags)
 	bytes_free=r->b_size- (int)(r->pos - r->buf);
 	
 	if (unlikely(bytes_free==0)){
-		LOG(L_ERR, "ERROR: tcp_read: buffer overrun, dropping\n");
+		LM_ERR("buffer overrun, dropping\n");
 		r->error=TCP_REQ_OVERRUN;
 		return -1;
 	}
@@ -373,11 +379,8 @@ int tcp_read_headers(struct tcp_connection *c, int* read_flags)
 	int bytes, remaining;
 	char *p;
 	struct tcp_req* r;
-
-#ifdef USE_STUN
 	unsigned int mc;   /* magic cookie */
 	unsigned short body_len;
-#endif
 
 #ifdef READ_MSRP
 	char *mfline;
@@ -439,7 +442,7 @@ int tcp_read_headers(struct tcp_connection *c, int* read_flags)
 		if (bytes<=0) return bytes;
 	}
 	p=r->parsed;
-	
+
 	while(p<r->pos && r->error==TCP_REQ_OK){
 		switch((unsigned char)r->state){
 			case H_BODY: /* read the body*/
@@ -487,6 +490,10 @@ int tcp_read_headers(struct tcp_connection *c, int* read_flags)
 					case '\n':
 						/* found LF LF */
 						r->state=H_BODY;
+#ifdef READ_HTTP11
+						if (cfg_get(tcp, tcp_cfg, accept_no_cl)!=0)
+							tcp_http11_continue(c);
+#endif
 						if (TCP_REQ_HAS_CLEN(r)){
 							r->body=p+1;
 							r->bytes_to_go=r->content_len;
@@ -496,9 +503,42 @@ int tcp_read_headers(struct tcp_connection *c, int* read_flags)
 								goto skip;
 							}
 						}else{
-							DBG("tcp_read_headers: ERROR: no clen, p=%X\n",
-									*p);
-							r->error=TCP_REQ_BAD_LEN;
+							if(cfg_get(tcp, tcp_cfg, accept_no_cl)!=0) {
+#ifdef READ_MSRP
+								/* if MSRP message */
+								if(c->req.flags&F_TCP_REQ_MSRP_FRAME)
+								{
+									r->body=p+1;
+									/* at least 3 bytes: 0\r\n */
+									r->bytes_to_go=3;
+									p++;
+									r->content_len = 0;
+									r->state=H_MSRP_BODY;
+									break;
+								}
+#endif
+
+#ifdef READ_HTTP11
+								if(TCP_REQ_BCHUNKED(r)) {
+									r->body=p+1;
+									/* at least 3 bytes: 0\r\n */
+									r->bytes_to_go=3;
+									p++;
+									r->content_len = 0;
+									r->state=H_HTTP11_CHUNK_START;
+									break;
+								}
+#endif
+								r->body=p+1;
+								r->bytes_to_go=0;
+								r->flags|=F_TCP_REQ_COMPLETE;
+								p++;
+								goto skip;
+							} else {
+								DBG("tcp_read_headers: ERROR: no clen, p=%X\n",
+										*p);
+								r->error=TCP_REQ_BAD_LEN;
+							}
 						}
 						break;
 					case '-':
@@ -610,18 +650,16 @@ int tcp_read_headers(struct tcp_connection *c, int* read_flags)
 						r->start=p;
 						break;
 					default:
-#ifdef USE_STUN
-						/* STUN support can be switched off even if it's compiled */
 						/* stun test */						
-						if (stun_allow_stun && (unsigned char)*p == 0x00) {
+						if (unlikely(sr_event_enabled(SREV_STUN_IN)) && (unsigned char)*p == 0x00) {
 							r->state=H_STUN_MSG;
 						/* body will used as pointer to the last used byte */
 							r->body=p;
 							r->content_len = 0;
 							DBG("stun msg detected\n");
-						}else
-#endif
-						r->state=H_SKIP;
+						} else {
+							r->state=H_SKIP;
+						}
 						r->start=p;
 				};
 				p++;
@@ -656,7 +694,7 @@ int tcp_read_headers(struct tcp_connection *c, int* read_flags)
 					r->state = H_SKIP_EMPTY;
 				}
 				break;
-#ifdef USE_STUN
+
 			case H_STUN_MSG:
 				if ((r->pos - r->body) >= sizeof(struct stun_hdr)) {
 					/* copy second short from buffer where should be body 
@@ -688,8 +726,8 @@ int tcp_read_headers(struct tcp_connection *c, int* read_flags)
 						}
 						else {
 							/* set content_len to length of fingerprint */
-							body_len = sizeof(struct stun_attr) + 
-									   SHA_DIGEST_LENGTH;
+							body_len = sizeof(struct stun_attr) + 20;
+							/* 20 is SHA_DIGEST_LENGTH from openssl/sha.h */
 						}
 					}
 					r->content_len=body_len;
@@ -711,7 +749,8 @@ int tcp_read_headers(struct tcp_connection *c, int* read_flags)
 					}
 					else {
 						/* set content_len to length of fingerprint */
-						body_len = sizeof(struct stun_attr)+SHA_DIGEST_LENGTH;
+						body_len = sizeof(struct stun_attr) + 20;
+						/* 20 is SHA_DIGEST_LENGTH from openssl/sha.h */
 						r->content_len=body_len;
 					}
 				}
@@ -736,7 +775,7 @@ int tcp_read_headers(struct tcp_connection *c, int* read_flags)
 					p = r->pos;
 				}
 				break;
-#endif /* USE_STUN */
+
 			change_state_case(H_CONT_LEN1,  'O', 'o', H_CONT_LEN2);
 			change_state_case(H_CONT_LEN2,  'N', 'n', H_CONT_LEN3);
 			change_state_case(H_CONT_LEN3,  'T', 't', H_CONT_LEN4);
@@ -805,17 +844,30 @@ int tcp_read_headers(struct tcp_connection *c, int* read_flags)
 					case '\r':
 					case ' ':
 					case '\t': /* FIXME: check if line contains only WS */
+						if(r->content_len<0) {
+							LM_ERR("bad Content-Length header value %d in"
+									" state %d\n", r->content_len, r->state);
+							r->content_len=0;
+							r->error=TCP_REQ_BAD_LEN;
+							r->state=H_SKIP; /* skip now */
+						}
 						r->state=H_SKIP;
 						r->flags|=F_TCP_REQ_HAS_CLEN;
 						break;
 					case '\n':
 						/* end of line, parse successful */
+						if(r->content_len<0) {
+							LM_ERR("bad Content-Length header value %d in"
+									" state %d\n", r->content_len, r->state);
+							r->content_len=0;
+							r->error=TCP_REQ_BAD_LEN;
+							r->state=H_SKIP; /* skip now */
+						}
 						r->state=H_LF;
 						r->flags|=F_TCP_REQ_HAS_CLEN;
 						break;
 					default:
-						LOG(L_ERR, "ERROR: tcp_read_headers: bad "
-								"Content-Length header value, unexpected "
+						LM_ERR("bad Content-Length header value, unexpected "
 								"char %c in state %d\n", *p, r->state);
 						r->state=H_SKIP; /* try to find another?*/
 				}
@@ -984,8 +1036,7 @@ int tcp_read_headers(struct tcp_connection *c, int* read_flags)
 #endif
 
 			default:
-				LOG(L_CRIT, "BUG: tcp_read_headers: unexpected state %d\n",
-						r->state);
+				LM_CRIT("unexpected state %d\n", r->state);
 				abort();
 		}
 	}
@@ -1019,6 +1070,132 @@ int msrp_process_msg(char* tcpbuf, unsigned int len,
 }
 #endif
 
+#ifdef READ_WS
+static int tcp_read_ws(struct tcp_connection *c, int* read_flags)
+{
+	int bytes, size, pos, mask_present;
+	unsigned int len;
+	char *p;
+	struct tcp_req *r;
+
+	r=&c->req;
+#ifdef USE_TLS
+	if (unlikely(c->type == PROTO_WSS))
+		bytes = tls_read(c, read_flags);
+	else
+#endif
+		bytes = tcp_read(c, read_flags);
+
+	if (bytes <= 0)
+	{
+		if (likely(r->parsed >= r->pos))
+			return 0;
+	}
+
+	size = r->pos - r->parsed;
+
+	p = r->parsed;
+	pos = 0;
+
+	/*
+	 0                   1                   2                   3
+	 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
+	+-+-+-+-+-------+-+-------------+-------------------------------+
+	|F|R|R|R| opcode|M| Payload len |    Extended payload length    |
+	|I|S|S|S|  (4)  |A|     (7)     |             (16/64)           |
+	|N|V|V|V|       |S|             |   (if payload len==126/127)   |
+	| |1|2|3|       |K|             |                               |
+	+-+-+-+-+-------+-+-------------+ - - - - - - - - - - - - - - - +
+	|     Extended payload length continued, if payload len == 127  |
+	+ - - - - - - - - - - - - - - - +-------------------------------+
+	|                               |Masking-key, if MASK set to 1  |
+	+-------------------------------+-------------------------------+
+	| Masking-key (continued)       |          Payload Data         |
+	+-------------------------------- - - - - - - - - - - - - - - - +
+	:                     Payload Data continued ...                :
+	+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - +
+	|                     Payload Data continued ...                |
+	+---------------------------------------------------------------+
+
+	Do minimal parse required to make sure the full message has been
+	received (websocket module will do full parse and validation).
+	*/
+
+	/* Process first two bytes */
+	if (size < pos + 2)
+		goto skip;
+	pos++;
+	mask_present = p[pos] & 0x80;
+	len = (p[pos++] & 0xff) & ~0x80;
+
+	/* Work out real length */
+	if (len == 126)
+	{
+		if (size < pos + 2)
+			goto skip;
+
+		len =	  ((p[pos + 0] & 0xff) <<  8)
+			| ((p[pos + 1] & 0xff) <<  0);
+		pos += 2;
+	}
+	else if (len == 127)
+	{
+		if (size < pos + 8)
+			goto skip;
+
+		/* Only decoding the last four bytes of the length...
+		   This limits the size of WebSocket messages that can be
+		   handled to 2^32 - which should be plenty for SIP! */
+		len =	  ((p[pos + 4] & 0xff) << 24)
+			| ((p[pos + 5] & 0xff) << 16)
+			| ((p[pos + 6] & 0xff) <<  8)
+			| ((p[pos + 7] & 0xff) <<  0);
+		pos += 8;
+	}
+
+	/* Skip mask */
+	if (mask_present)
+	{
+		if (size < pos + 4)
+			goto skip;
+		pos += 4;
+	}
+
+	/* Now check the whole message has been received */
+	if (size < pos + len)
+		goto skip;
+
+	pos += len;
+	r->flags |= F_TCP_REQ_COMPLETE;
+	r->parsed = &p[pos];
+
+skip:
+	return bytes;
+}
+
+static int ws_process_msg(char* tcpbuf, unsigned int len,
+		struct receive_info* rcv_info, struct tcp_connection* con)
+{
+	int ret;
+	tcp_event_info_t tev;
+
+	ret = 0;
+	LM_DBG("WebSocket Message: [[>>>\n%.*s<<<]]\n", len, tcpbuf);
+	if(likely(sr_event_enabled(SREV_TCP_WS_FRAME_IN))) {
+		memset(&tev, 0, sizeof(tcp_event_info_t));
+		tev.type = SREV_TCP_WS_FRAME_IN;
+		tev.buf = tcpbuf;
+		tev.len = len;
+		tev.rcv = rcv_info;
+		tev.con = con;
+		ret = sr_event_exec(SREV_TCP_WS_FRAME_IN, (void*)(&tev));
+	} else {
+		LM_DBG("no callback registering for handling WebSockets - dropping!\n");
+	}
+	return ret;
+}
+#endif
+
 /**
  * @brief wrapper around receive_msg() to clone the tcpbuf content
  *
@@ -1045,6 +1222,11 @@ int receive_tcp_msg(char* tcpbuf, unsigned int len,
 		if(unlikely(con->req.flags&F_TCP_REQ_MSRP_FRAME))
 			return msrp_process_msg(tcpbuf, len, rcv_info, con);
 #endif
+#ifdef READ_WS
+		if(unlikely(con->type == PROTO_WS || con->type == PROTO_WSS))
+			return ws_process_msg(tcpbuf, len, rcv_info, con);
+#endif
+
 		return receive_msg(tcpbuf, len, rcv_info);
 	}
 
@@ -1087,12 +1269,20 @@ int receive_tcp_msg(char* tcpbuf, unsigned int len,
 #ifdef READ_MSRP
 	if(unlikely(con->req.flags&F_TCP_REQ_MSRP_FRAME))
 		return msrp_process_msg(buf, len, rcv_info, con);
+#endif
+#ifdef READ_WS
+	if(unlikely(con->type == PROTO_WS || con->type == PROTO_WSS))
+		return ws_process_msg(buf, len, rcv_info, con);
 #endif
 	return receive_msg(buf, len, rcv_info);
 #else /* TCP_CLONE_RCVBUF */
 #ifdef READ_MSRP
 	if(unlikely(con->req.flags&F_TCP_REQ_MSRP_FRAME))
 		return msrp_process_msg(tcpbuf, len, rcv_info, con);
+#endif
+#ifdef READ_WS
+	if(unlikely(con->type == PROTO_WS || con->type == PROTO_WSS))
+		return ws_process_msg(tcpbuf, len, rcv_info, con);
 #endif
 	return receive_msg(tcpbuf, len, rcv_info);
 #endif /* TCP_CLONE_RCVBUF */
@@ -1116,7 +1306,12 @@ int tcp_read_req(struct tcp_connection* con, int* bytes_read, int* read_flags)
 
 again:
 		if (likely(req->error==TCP_REQ_OK)){
-			bytes=tcp_read_headers(con, read_flags);
+#ifdef READ_WS
+			if (unlikely(con->type == PROTO_WS || con->type == PROTO_WSS))
+				bytes=tcp_read_ws(con, read_flags);
+			else
+#endif
+				bytes=tcp_read_headers(con, read_flags);
 #ifdef EXTRA_DEBUG
 						/* if timeout state=0; goto end__req; */
 			DBG("read= %d bytes, parsed=%d, state=%d, error=%d\n",
@@ -1144,13 +1339,12 @@ again:
 				resp=CONN_EOF;
 				goto end_req;
 			}
-		
 		}
 		if (unlikely(req->error!=TCP_REQ_OK)){
-			LOG(L_ERR,"ERROR: tcp_read_req: bad request, state=%d, error=%d "
-					  "buf:\n%.*s\nparsed:\n%.*s\n", req->state, req->error,
-					  (int)(req->pos-req->buf), req->buf,
-					  (int)(req->parsed-req->start), req->start);
+			LM_ERR("bad request, state=%d, error=%d buf:\n%.*s\nparsed:\n%.*s\n",
+					req->state, req->error,
+					(int)(req->pos-req->buf), req->buf,
+					(int)(req->parsed-req->start), req->start);
 			DBG("- received from: port %d\n", con->rcv.src_port);
 			print_ip("- received from: ip ",&con->rcv.src_ip, "\n");
 			resp=CONN_ERROR;
@@ -1172,8 +1366,7 @@ again:
 			}else{
 				if (cfg_get(tcp, tcp_cfg, accept_no_cl)==0) {
 					req->error=TCP_REQ_BAD_LEN;
-					LOG(L_ERR, "ERROR: tcp_read_req: content length not present or"
-						" unparsable\n");
+					LM_ERR("content length not present or unparsable\n");
 					resp=CONN_ERROR;
 					goto end_req;
 				}
@@ -1204,17 +1397,14 @@ again:
 				init_dst_from_rcv(&dst, &con->rcv);
 
 				if (tcp_send(&dst, 0, CRLF, CRLF_LEN) < 0) {
-					LOG(L_ERR, "CRLF ping: tcp_send() failed\n");
+					LM_ERR("CRLF ping: tcp_send() failed\n");
 				}
 				ret = 0;
-			}else
-#ifdef USE_STUN
-			if (unlikely(req->state==H_STUN_END)){
+			} else if (unlikely(req->state==H_STUN_END)) {
 				/* stun request */
 				ret = stun_process_msg(req->start, req->parsed-req->start,
 									 &con->rcv);
-			}else
-#endif
+			} else
 #ifdef READ_MSRP
 			// if (unlikely(req->flags&F_TCP_REQ_MSRP_FRAME)){
 			if (unlikely(req->state==H_MSRP_FINISH)){
@@ -1231,6 +1421,12 @@ again:
 						req->body + req->content_len - req->start,
 						&con->rcv, con);
 			}else
+#endif
+#ifdef READ_WS
+			if (unlikely(con->type == PROTO_WS || con->type == PROTO_WSS)){
+				ret = receive_tcp_msg(req->start, req->parsed-req->start,
+									&con->rcv, con);
+			}else
 #endif
 				ret = receive_tcp_msg(req->start, req->parsed-req->start,
 									&con->rcv, con);
@@ -1295,7 +1491,7 @@ void release_tcpconn(struct tcp_connection* c, long state, int unix_sock)
 		response[1]=state;
 		
 		if (tsend_stream(unix_sock, (char*)response, sizeof(response), -1)<=0)
-			LOG(L_ERR, "ERROR: release_tcpconn: tsend_stream failed\n");
+			LM_ERR("tsend_stream failed\n");
 }
 
 
@@ -1313,7 +1509,7 @@ static ticks_t tcpconn_read_timeout(ticks_t t, struct timer_ln* tl, void* data)
 	}
 	/* if conn->state is ERROR or BAD => force timeout too */
 	if (unlikely(io_watch_del(&io_w, c->fd, -1, IO_FD_CLOSING)<0)){
-		LOG(L_ERR, "ERROR: tcpconn_read_timeout: io_watch_del failed for %p"
+		LM_ERR("io_watch_del failed for %p"
 					" id %d fd %d, state %d, flags %x, main fd %d\n",
 					c, c->id, c->fd, c->state, c->flags, c->s);
 	}
@@ -1362,37 +1558,33 @@ again:
 					break;
 				}else if (errno == EINTR) goto again;
 				else{
-					LOG(L_CRIT,"BUG: tcp_receive: handle_io: read_fd: %s \n",
-							strerror(errno));
+					LM_CRIT("read_fd: %s \n", strerror(errno));
 						abort(); /* big error*/
 				}
 			}
 			if (unlikely(n==0)){
-				LOG(L_ERR, "WARNING: tcp_receive: handle_io: 0 bytes read\n");
+				LM_ERR("0 bytes read\n");
 				goto error;
 			}
 			if (unlikely(con==0)){
-					LOG(L_CRIT, "BUG: tcp_receive: handle_io null pointer\n");
+					LM_CRIT("null pointer\n");
 					goto error;
 			}
 			con->fd=s;
 			if (unlikely(s==-1)) {
-				LOG(L_ERR, "ERROR: tcp_receive: handle_io: read_fd:"
-									"no fd read\n");
+				LM_ERR("read_fd: no fd read\n");
 				goto con_error;
 			}
 			con->reader_pid=my_pid();
 			if (unlikely(con==tcp_conn_lst)){
-				LOG(L_CRIT, "BUG: tcp_receive: handle_io: duplicate"
-							" connection received: %p, id %d, fd %d, refcnt %d"
+				LM_CRIT("duplicate connection received: %p, id %d, fd %d, refcnt %d"
 							" state %d (n=%d)\n", con, con->id, con->fd,
 							atomic_get(&con->refcnt), con->state, n);
 				goto con_error;
 				break; /* try to recover */
 			}
 			if (unlikely(con->state==S_CONN_BAD)){
-				LOG(L_WARN, "WARNING: tcp_receive: handle_io: received an"
-							" already bad connection: %p id %d refcnt %d\n",
+				LM_WARN("received an already bad connection: %p id %d refcnt %d\n",
 							con, con->id, atomic_get(&con->refcnt));
 				goto con_error;
 			}
@@ -1432,8 +1624,7 @@ repeat_1st_read:
 			local_timer_add(&tcp_reader_ltimer, &con->timer,
 								S_TO_TICKS(TCP_CHILD_TIMEOUT), t);
 			if (unlikely(io_watch_add(&io_w, s, POLLIN, F_TCPCONN, con)<0)){
-				LOG(L_CRIT, "ERROR: tcpconn_receive: handle_io: io_watch_add "
-							"failed for %p id %d fd %d, state %d, flags %x,"
+				LM_CRIT("io_watch_add failed for %p id %d fd %d, state %d, flags %x,"
 							" main fd %d, refcnt %d\n",
 							con, con->id, con->fd, con->state, con->flags,
 							con->s, atomic_get(&con->refcnt));
@@ -1447,8 +1638,7 @@ repeat_1st_read:
 			if (unlikely(con->state==S_CONN_BAD)){
 				resp=CONN_ERROR;
 				if (!(con->send_flags.f & SND_F_CON_CLOSE))
-					LOG(L_WARN, "WARNING: tcp_receive: handle_io: F_TCPCONN"
-							" connection marked as bad: %p id %d refcnt %d\n",
+					LM_WARN("F_TCPCONN connection marked as bad: %p id %d refcnt %d\n",
 							con, con->id, atomic_get(&con->refcnt));
 				goto read_error;
 			}
@@ -1468,8 +1658,7 @@ read_error:
 				ret=-1; /* some error occured */
 				if (unlikely(io_watch_del(&io_w, con->fd, idx,
 											IO_FD_CLOSING) < 0)){
-					LOG(L_CRIT, "ERROR: tcpconn_receive: handle_io: "
-							"io_watch_del failed for %p id %d fd %d,"
+					LM_CRIT("io_watch_del failed for %p id %d fd %d,"
 							" state %d, flags %x, main fd %d, refcnt %d\n",
 							con, con->id, con->fd, con->state,
 							con->flags, con->s, atomic_get(&con->refcnt));
@@ -1493,12 +1682,12 @@ read_error:
 			}
 			break;
 		case F_NONE:
-			LOG(L_CRIT, "BUG: handle_io: empty fd map %p (%d): "
-						"{%d, %d, %p}\n", fm, (int)(fm-io_w.fd_hash),
+			LM_CRIT("empty fd map %p (%d): {%d, %d, %p}\n",
+						fm, (int)(fm-io_w.fd_hash),
 						fm->fd, fm->type, fm->data);
 			goto error;
 		default:
-			LOG(L_CRIT, "BUG: handle_io: uknown fd type %d\n", fm->type); 
+			LM_CRIT("uknown fd type %d\n", fm->type); 
 			goto error;
 	}
 	
@@ -1538,8 +1727,7 @@ void tcp_receive_loop(int unix_sock)
 		goto error;
 	/* add the unix socket */
 	if (io_watch_add(&io_w, tcpmain_sock, POLLIN,  F_TCPMAIN, 0)<0){
-		LOG(L_CRIT, "ERROR: tcp_receive_loop: init: failed to add socket "
-							" to the fd list\n");
+		LM_CRIT("failed to add socket to the fd list\n");
 		goto error;
 	}
 
@@ -1601,20 +1789,17 @@ void tcp_receive_loop(int unix_sock)
 			break;
 #endif
 		default:
-			LOG(L_CRIT, "BUG: tcp_receive_loop: no support for poll method "
-					" %s (%d)\n", 
+			LM_CRIT("no support for poll method %s (%d)\n", 
 					poll_method_name(io_w.poll_method), io_w.poll_method);
 			goto error;
 	}
 error:
 	destroy_io_wait(&io_w);
-	LOG(L_CRIT, "ERROR: tcp_receive_loop: exiting...");
+	LM_CRIT("exiting...");
 	exit(-1);
 }
 
 
-
-#ifdef USE_STUN
 int is_msg_complete(struct tcp_req* r)
 {
 	if (TCP_REQ_HAS_CLEN(r)) {
@@ -1629,6 +1814,5 @@ int is_msg_complete(struct tcp_req* r)
 		return 1;
 	}
 }
-#endif
 
 #endif /* USE_TCP */
